--[[
Minim - 2014, Ulysse Ramage

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

The Software is provided “as is”, without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, fitness for a particular purpose and noninfringement. In no event shall the author or copyright holder Ulysse Ramage be liable for any claim, damages or other liability, whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software or the use or other dealings in the Software.
]]--

http = require("socket.http")
downloader = require('/include/LOVEDownloader')
slam = require("/include/slam")
flux = require("/include/flux")
cron = require("/include/cron")
require("/include/player")
require("/include/gravitySwitch")
require("/include/enemy")

-----------------
------VALUES-----
-----------------

WWIDTH, WHEIGHT = love.window.getDesktopDimensions();
love.window.setMode(WWIDTH, WHEIGHT)

scaleFactor = WHEIGHT/480

atariFont = {small = love.graphics.newFont("fonts/atari.ttf", 15*scaleFactor), big = love.graphics.newFont("/fonts/atari.ttf", 42*scaleFactor), huge = love.graphics.newFont("/fonts/atari.ttf", 84*scaleFactor)}

love.math.setRandomSeed(os.time())

joystickKeys = {
  left = 14,
  right = 15,
  a = 1
}
joysticks = love.joystick.getJoysticks()

hue = 0

onlineMode = false

-----------------
----FUNCTIONS----
-----------------

loadOnlineScores = love.filesystem.load("highscore.lua")

function sendScore(name, score) http.request("http://graphene.net84.net/swotch/addScore.php?name="..name.."&score="..score); end

function randomString( length )
  local string = ""
  local letters = {
  "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"
  }
  for i = 1, length do
    string = string .. letters[love.math.random(1, #letters)]
  end
  return string
end

function drawAngleRectangle( mode, x, y, w, h, a, ox, oy )
  ox = ox or 0
  oy = oy or 0
  a = a or 0
  love.graphics.push()
  love.graphics.translate( x, y )
  love.graphics.rotate( a )
  love.graphics.rectangle( mode,-ox, -oy, w, h )
  love.graphics.pop()
end

function distanceBetween(x1,y1,x2,y2) return math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2) end

function HSL(h, s, l, a)
  if s<=0 then return l,l,l,a end
  h, s, l = h/256*6, s/255, l/255
  local c = (1-math.abs(2*l-1))*s
  local x = (1-math.abs(h%2-1))*c
  local m,r,g,b = (l-.5*c), 0,0,0
  if h < 1     then r,g,b = c,x,0
  elseif h < 2 then r,g,b = x,c,0
  elseif h < 3 then r,g,b = 0,c,x
  elseif h < 4 then r,g,b = 0,x,c
  elseif h < 5 then r,g,b = x,0,c
  else              r,g,b = c,0,x
  end return (r+m)*255,(g+m)*255,(b+m)*255,a
end

function spawnEnemy(spawnDelay)
  local side = love.math.random(0, 1) * 2 - 1
  local enemy = Enemy:new(WWIDTH*0.5+(WWIDTH*0.6*side), love.math.random(50, WHEIGHT-50), 25*scaleFactor, 25*scaleFactor)
  enemy.side = side
  if love.math.random(1, 10) == 1 then
    enemy.isPowerup = true
    enemy.powerup = "slowMoLoad"
    enemy.speed = 2
    enemy.width = 50*scaleFactor
  else
    enemy.speed = love.math.random(3, 6)
  end
  if spawnDelay > 0.5 then
    spawnDelay = spawnDelay - 0.05
  end
  spawnTimer = cron.every(spawnDelay, function() spawnEnemy(spawnDelay); end)
  objects.enemies[#objects.enemies+1] = enemy
end

function slowMoFunc(event)
  if event == "press" then
    slowMo.isPressing = true
    shake.value = shake.originValue * 0.25
    slowMo.fade = "out"
    sounds["slowMoIn"]:play()
  elseif event == "release" then
    slowMo.isPressing = false
    shake.value = shake.originValue
    slowMo.fade = "in"
    sounds["slowMoOut"]:play()
  end
end

function drawLogo()
  if logo.alpha > 0 then
    love.graphics.setFont(atariFont.huge)
    love.graphics.setColor(HSL(hue, 150+hueIntensity, 150+hueIntensity, logo.alpha))
    love.graphics.printf("SW", logo.left.x, WHEIGHT*0.435, 100*scaleFactor, "right")
    love.graphics.printf("TCH", logo.right.x, WHEIGHT*0.435, 100*scaleFactor, "left")
  end
end

function drawUI()
  if ui.alpha > 0 then
    love.graphics.setFont(atariFont.big)
    love.graphics.setColor(HSL(hue, 70+hueIntensity, 70+hueIntensity, ui.alpha))
    if highscore > 0 then
      local uiTopY
      if onlineMode == true then love.graphics.printf(name, 0, ui.top.y-10, WWIDTH, "center"); uiTopY = ui.top.y + 30 else uiTopY = ui.top.y end
      love.graphics.printf(highscore, 0, uiTopY, WWIDTH, "center")
    end
    if onlineMode == true and score3 > 0 then
      love.graphics.setColor(HSL(hue, 100+hueIntensity, 100+hueIntensity, ui.alpha))
      love.graphics.printf(name1, 0, ui.bottom.y-60, WWIDTH/3, "center")
      love.graphics.printf(score1, 0, ui.bottom.y-20, WWIDTH/3, "center")
      love.graphics.printf(name2, WWIDTH/3, ui.bottom.y-60, WWIDTH/3, "center")
      love.graphics.printf(score2, WWIDTH/3, ui.bottom.y-20, WWIDTH/3, "center")
      love.graphics.printf(name3, WWIDTH/3*2, ui.bottom.y-60, WWIDTH/3, "center")
      love.graphics.printf(score3, WWIDTH/3*2, ui.bottom.y-20, WWIDTH/3, "center")
    end
  end
end

function gameEvent(event)
  if event == "start" then
    sounds["backgroundTheme"]:stop()
    setAudio()
    sounds["backgroundTheme"]:stop()
    sounds["backgroundTheme"]:setPitch(1)
    sounds["backgroundTheme"]:play()
    hasGameStarted = true
    canReplay = false
    hueIntensity = 50
    spawnTimer = cron.every(spawnDelay, function() spawnEnemy(spawnDelay); end)
    flux.remove(ui)
    flux.remove(ui.top)
    flux.to(logo, 0.5, {alpha = 0}):ease("quadout")
    flux.to(logo.left, 0.5, {x = -WWIDTH*0.5}):ease("quadout")
    flux.to(logo.right, 0.5, {x = WWIDTH}):ease("quadout")
    flux.to(ui, 0.25, {alpha = 0}):ease("quadout")
    flux.to(ui.top, 0.25, {y = -10}):ease("quadout")
    flux.to(ui.bottom, 0.25, {y = WHEIGHT+10}):ease("quadout")
  elseif event == "end" then
    
    local tempScore = math.floor(score)
    
    if math.floor(score) > highscore or highscore == 0 then
      love.filesystem.write("data.lua", "name = \"" .. name .. "\"\nhighscore = " .. math.floor(score))
      highscore = math.floor(score)
    end
    
    gameOver.value = true
    if slowMo.isPressing == true then gameOver.number = slowMo.value/100 end
    slowMo.isPressing = true
    slowMo.fade = "out"
    flux.to(shake, 0.5, {value = 0}):oncomplete(function()
       hasGameStarted = false
       canReplay = false
       for i = 1, #objects.enemies do
         if objects.enemies[i].isDead == false then
           flux.to(objects.enemies[i], 2, {x = objects.enemies[i]:getX()+WWIDTH*2}):oncomplete(function() objects.enemies[i].isDead = true; end)
         end
       end
       flux.to(objects.player, 1, {x = objects.player.xOrigin, y = objects.player.yOrigin, rotation = 0}):ease("quadout")
       flux.to(logo.left, 1, {x = logo.left.xOrigin}):ease("quadout")
       flux.to(logo.right, 1, {x = logo.right.xOrigin}):ease("quadout")
       flux.to(logo, 1, {alpha = logo.alphaOrigin}):ease("quadout"):oncomplete(function()
         --sendScore(name, math.floor(tempScore))
         flux.to(ui, 0.25, {alpha = 255}):ease("quadout")
         flux.to(ui.top, 0.25, {y = ui.top.yOrigin}):ease("quadout")
         flux.to(ui.bottom, 0.25, {y = ui.bottom.yOrigin}):ease("quadout")
         canReplay = true;
       end)
       --reset everything
       setVariables()
       spawnTimer:reset()
       sounds["backgroundTheme"]:setVolume(1)
       sounds["backgroundTheme"]:setPitch(0.1)
    end)
    sounds["slowMoIn"]:setPitch(3)
    sounds["slowMoIn"]:play()
    sounds["explosion"]:play()
    slowMo.fade = "out"
  end
end

function onCollide(enemyType, enemy)
  if enemyType == "enemy" then
    if gameOver.value == false then gameEvent("end") end
  elseif enemyType == "slowMoLoad" and enemy.isDying == false then
    enemy.isDying = true
    if slowMo.value < slowMo.origin then slowMo.isLoading = true; elseif slowMo.value == 100 then score = score + 250 end
    slowMoFunc("release")
    flux.to(enemy, 0.1, {width = 0.1}):oncomplete(function() enemy.isDead = true end)
  end
end

-----------------
------SOUNDS-----
-----------------

sounds = {
  ["backgroundTheme"] = love.audio.newSource("/audio/theme.wav", "stream"),
  ["explosion"] = love.audio.newSource("/audio/explosion.wav"),
  ["slowMoIn"] = love.audio.newSource("/audio/slowMoIn.wav"),
  ["slowMoOut"] = love.audio.newSource("/audio/slowMoOut.wav")
}

-----------------
---INITIALIZING--
-----------------

isTouching = {
  left = false,
  right = false,
  slowMo = false,
  joyID = nil,
  slowMoID = nil
}

logo = {
    left = { x = WWIDTH*0.5-40*scaleFactor-45*scaleFactor-100*scaleFactor },
    right = { x = WWIDTH*0.5-40*scaleFactor-115*scaleFactor*0.5+100*scaleFactor },
    alpha = 255
  }
  logo.left.xOrigin = logo.left.x
  logo.right.xOrigin = logo.right.x
  logo.alphaOrigin = logo.alpha
  
  ui = {
    top = { y = 25 },
    bottom = { y = WHEIGHT-25 },
    alpha = 255
  }
  ui.top.yOrigin = ui.top.y
  ui.bottom.yOrigin = ui.bottom.y
  ui.alphaOrigin = ui.alpha

function setVariables()

  shake = {originValue = 4, value = 4}

  gDir = 1
  gravityLim = 1400*scaleFactor
  gravity = gravityLim

  hueIntensity = 0

  score = 0
  
  spawnDelay = 2

  slowMo = {
    origin = 150,
    value = 150,
    isPressing = false,
    isLoading = false,
    fade = false,
    volume = 1
  }

  hasGameStarted = false
  gameOver = {value = false, number = 1}
  
  if onlineMode == true then loadOnlineScores() end

end

function setAudio()
  sounds["backgroundTheme"]:setLooping( true )
  sounds["backgroundTheme"]:setVolume(1)
  sounds["backgroundTheme"]:setPitch(0.1)
  sounds["backgroundTheme"]:play()
  sounds["slowMoIn"]:setPitch(1)
  sounds["explosion"]:setVolume(0.5)
end

setVariables()
canReplay = true
setAudio()

highscore = 0

name = randomString( 3 )

if onlineMode == true then

  name1 = "AAA"
  score1 = 300
  name2 = "BBB"
  score2 = 200
  name3 = "CCC"
  score3 = 0

  highscoreFile = downloader.download("http://graphene.net84.net/swotch/addScore.php", "highscore.lua")
  highscoreFile:start() -- Start download
  highscoreFile.callbacks["success"] = function() highscoreFile.success = true end

end

loadData = love.filesystem.load("data.lua")
if loadData then loadData() end

-----------------
----INSTANCES----
-----------------

objects = {
  player = Player:new(WWIDTH*0.5-40*scaleFactor, WHEIGHT*0.5, 75*scaleFactor, 1000),
  topSwitch = GravitySwitch:new(WWIDTH*0.5, 0, WWIDTH, 10),
  bottomSwitch = GravitySwitch:new(WWIDTH*0.5, WHEIGHT, WWIDTH, 10),
  leftBound = GravitySwitch:new(0, WHEIGHT*0.5, 10, WHEIGHT),
  rightBound = GravitySwitch:new(WWIDTH, WHEIGHT*0.5, 10, WHEIGHT),
  enemies = {}
}

objects.player.yOrigin = objects.player.y

-----------------
------CODE !-----
-----------------

function love.update(dt)
  
  if onlineMode == "true" then
    if highscoreFile.success then
    
      highscoreFile.ignore = true
      loadOnlineScores()
      
    else
      
      highscoreFile:update()
      
    end
  end

  local dspeed = dt
  
  if (love.keyboard.isDown(" ") or (joysticks[1] and joysticks[1]:isDown(joystickKeys.a)) or (isTouching.slowMo)) and gameOver.value == false and hasGameStarted == true and slowMo.isLoading == false and slowMo.value > 0 then
    
    if slowMo.isPressing == false then slowMoFunc("press") end
    
    dspeed = dspeed * 0.25
    
    slowMo.value = slowMo.value - dt*50
    
  elseif hasGameStarted == true and gameOver.value == false and slowMo.value < slowMo.origin and slowMo.isLoading == true then
    
    slowMo.value = slowMo.value + dt*100
    
  end
  
  flux.update(dspeed)
  
  if gameOver.value == true then
    if gameOver.number > 0 then gameOver.number = gameOver.number - dt*4 end
    dspeed = dspeed*gameOver.number
  end
  
  if slowMo.fade == "in" or slowMo.fade == "out" then
    if slowMo.fade == "in" and slowMo.volume < 1 then
      slowMo.volume = slowMo.volume + dt
      if slowMo.volume > 1 then slowMo.volume = 1 end
    elseif slowMo.fade == "out" and slowMo.volume > 0.1 then
      slowMo.volume = slowMo.volume - dt
      if slowMo.volume < 0.1 then slowMo.volume = 0.1 end
    end
  sounds["backgroundTheme"]:setVolume(slowMo.volume)
  sounds["backgroundTheme"]:setPitch(slowMo.volume)
  end
  
  if slowMo.value > slowMo.origin and slowMo.isLoading == true then slowMo.value = slowMo.origin; slowMo.isLoading = false end
  if slowMo.value < 0 then slowMo.value = 0 end
  if slowMo.value == 0 and (love.keyboard.isDown(" ") or (joysticks[1] and joysticks[1]:isDown(joystickKeys.a)) or (isTouching.slowMo)) and gameOver.value == false and slowMo.isPressing == true then slowMoFunc("release") end

  if spawnTimer and hasGameStarted == true and dspeed > 0 then spawnTimer:update(dspeed) end

  if hue > 255 then hue = 0
  elseif hue < 0 then hue = 255 end
  hue = hue +100*dspeed

  if hasGameStarted == true and gameOver.number > 0 then

    score = score+dspeed*100
    objects.player.rotation = math.rad(math.floor(math.deg(objects.player.rotation+dt*8)))

    if #objects.enemies > 0 then
      for i = 1, #objects.enemies do
        if objects.enemies[i].isDead == false then
          objects.enemies[i].rotation = objects.enemies[i].rotation + love.math.random(50, 100)*dt
          objects.enemies[i]:setX(objects.enemies[i]:getX()-objects.enemies[i].side*dspeed*100*objects.enemies[i].speed*scaleFactor)
          if distanceBetween(objects.player:getX(), objects.player:getY(), objects.enemies[i]:getX(), objects.enemies[i]:getY()) < objects.player.width and gameOver.value == false then
            if objects.enemies[i].isPowerup == true then
              if objects.enemies[i].powerup == "slowMoLoad" then onCollide("slowMoLoad", objects.enemies[i]); end
            else
              onCollide("enemy")
            end
          elseif (objects.enemies[i]:getX() > WWIDTH+objects.enemies[i].width*0.5 and objects.enemies[i].side == -1) or (objects.enemies[i]:getX() < -objects.enemies[i].width*0.5 and objects.enemies[i].side == 1) and gameOver.value == false then
            objects.enemies[i].isDead = true
          end
        end
      end
    end

    if love.keyboard.isDown("left") or love.keyboard.isDown("q") or love.keyboard.isDown("a") or (joysticks[1] and joysticks[1]:isDown(joystickKeys.left)) or (isTouching.left) then
      objects.player:setX(objects.player:getX() - objects.player:getSpeed()*dspeed*scaleFactor)
    elseif love.keyboard.isDown("right") or love.keyboard.isDown("d") or (joysticks[1] and joysticks[1]:isDown(joystickKeys.right)) or (isTouching.right) then
      objects.player:setX(objects.player:getX() + objects.player:getSpeed()*dspeed*scaleFactor)
    end
    
    if objects.player:getX() - objects.player:getWidth()*0.5 < objects.leftBound:getX() then
      objects.player:setX(objects.leftBound:getX()+objects.player:getWidth()*0.5*scaleFactor)
    elseif objects.player:getX() + objects.player:getWidth()*0.5 > objects.rightBound:getX() then
      objects.player:setX(objects.rightBound:getX()-objects.player:getWidth()*0.5*scaleFactor)
    end
    
    if gravity > -gravityLim and gDir == -1 then
      gravity = gravity - gravityLim*0.2*dspeed*100
    elseif gravity < gravityLim and gDir == 1 then
      gravity = gravity + gravityLim*0.2*dspeed*100
    end
    
    objects.player:setY(objects.player:getY() + gravity*dspeed)
  
    if (objects.player:getY() - objects.player:getWidth()*0.5 < objects.topSwitch:getY()) then
      gDir = 1
    elseif (objects.player:getY() + objects.player:getWidth()*0.5 > objects.bottomSwitch:getY()) then
      gDir = -1
    end
  
  end

  
end

function love.draw()
  
  love.graphics.push()
  
  if hasGameStarted == false then
    
    love.graphics.translate(love.math.random(-shake.value*0.25, shake.value*0.25), love.math.random(-shake.value*0.25, shake.value*0.25))
    
    love.graphics.push()
    love.graphics.translate(love.math.random(-shake.value*0.25, shake.value*0.25), love.math.random(-shake.value*0.25, shake.value*0.25))
    drawLogo()
    love.graphics.pop()
    
    drawUI()

  else

    love.graphics.translate(love.math.random(-shake.value, shake.value), love.math.random(-shake.value, shake.value))
    
    drawLogo()
    
    drawUI()
    
    love.graphics.translate(WWIDTH*0.5, WHEIGHT*0.5)
    love.graphics.rotate(math.rad(love.math.random(-shake.value*0.5, shake.value*0.5)))
    love.graphics.translate(-WWIDTH*0.5, -WHEIGHT*0.5)
    
    love.graphics.setFont(atariFont.huge)
    love.graphics.setColor(HSL(hue, 50+hueIntensity, 50+hueIntensity))
    
    if hasGameStarted == true and canReplay == false then
    
      love.graphics.push()
      love.graphics.translate(love.math.random(-shake.value*0.5, shake.value*0.5), love.math.random(-shake.value*0.5, shake.value*0.5))
      love.graphics.printf(math.floor(score), 0, WHEIGHT*0.35, WWIDTH, "center")
      if slowMo.value < slowMo.origin and slowMo.isLoading == true then love.graphics.setColor(HSL(hue, 20+hueIntensity, 20+hueIntensity)) end
      love.graphics.rectangle("fill", WWIDTH*0.5-250*scaleFactor, WHEIGHT*0.55, 500*(slowMo.value/slowMo.origin)*scaleFactor, 40*scaleFactor)
      love.graphics.pop()

    end
    
    drawLogo()
    
  end
  
  love.graphics.setColor(HSL(hue, 100+hueIntensity, 100+hueIntensity))
  objects.topSwitch:draw()
  objects.bottomSwitch:draw()
  objects.leftBound:draw()
  objects.rightBound:draw()
  
  if #objects.enemies > 0 then
    for i = 1, #objects.enemies do
      if objects.enemies[i].isDead == false then
        objects.enemies[i]:draw()
      end
    end
  end
  
  love.graphics.setColor(HSL(hue, 150+hueIntensity, 150+hueIntensity))
  objects.player:draw()
  
  love.graphics.pop()
  
  love.graphics.setBackgroundColor(HSL(hue, 40+hueIntensity*0.5, 40+hueIntensity*0.5))

end

function love.keypressed(key, isrepeat)
  if hasGameStarted == false and canReplay == true and key ~= "escape" then
    gameEvent("start")
  end
  
  if key == " " and gameOver.value == false and slowMo.isLoading == false and slowMo.value > 0 and hasGameStarted == true then
    slowMoFunc("press")
  end
end

function love.keyreleased(key)
  if key == "escape" then love.event.quit() end
  
  if key == " " and gameOver.value == false and slowMo.isPressing == true and hasGameStarted == true then
    slowMoFunc("release")
  end
end

function love.joystickpressed(joystick, key)
  if (key == joystickKeys.left or key == joystickKeys.right or key == joystickKeys.a) and hasGameStarted == false and canReplay == true then
    gameEvent("start")
  end
  
  if key == joystickKeys.a and gameOver.value == false and slowMo.isLoading == false and slowMo.value > 0 and hasGameStarted == true then
    slowMoFunc("press")
  end
end

function love.joystickreleased(joystick, key)
  if key == joystickKeys.a and gameOver.value == false and slowMo.isPressing == true and hasGameStarted == true then
    slowMoFunc("release")
  end
end

function love.joystickadded()
  joysticks = love.joystick.getJoysticks()
end

function love.joystickremoved()
  joysticks = love.joystick.getJoysticks()
end

-----------------
------PHONE------
-----------------

function love.touchpressed( id, x, y, pressure )
  x, y = x*WWIDTH, y*WHEIGHT
  
  if hasGameStarted == false and canReplay == true then gameEvent("start") end
  
  if x < 100 and isTouching.joyID == nil then --touched left
    isTouching.left = true; isTouching.joyID = id
  elseif x > 150 and x < 250 and isTouching.joyID == nil then --touched right
    isTouching.right = true; isTouching.joyID = id
  elseif x > WWIDTH-200 and isTouching.slowMoID == nil then --touched slowMo
    slowMoFunc("press")
    isTouching.slowMo = true
    isTouching.slowMoID = id
  end
end

function love.touchreleased( id, x, y, pressure )
  x, y = x*WWIDTH, y*WHEIGHT
  
  if id == isTouching.joyID then isTouching.joyID = nil; isTouching.left = false; isTouching.right = false
  elseif id == isTouching.slowMoID then isTouching.slowMoID = nil; slowMoFunc("release"); isTouching.slowMo = false end

end

function love.touchmoved( id, x, y, pressure )
  x, y = x*WWIDTH, y*WHEIGHT
  
  if x < 100 and id == isTouching.joyID then --touched left
    isTouching.left = true; isTouching.right = false;
  elseif x > 150 and x < 250 and id == isTouching.joyID then --touched right
    isTouching.left = false; isTouching.right = true;
  elseif x >= 100 and x <= 150 and id == isTouching.joyID then
    isTouching.left = false; isTouching.right = false;
  end
end